@@ -17,45 +17,90 @@
 /// SQL Abstract Syntax Tree (AST)
 #[derive(Debug, Clone, PartialEq)]
 pub enum ASTNode {
+    /// Identifier e.g. table name or column name
     SQLIdentifier(String),
+    /// Wildcard e.g. `*`
     SQLWildcard,
+    /// Multi part identifier e.g. `myschema.dbo.mytable`
     SQLCompoundIdentifier(Vec<String>),
+    /// Assigment e.g. `name = 'Fred'` in an UPDATE statement
+    SQLAssignment(String, Box<ASTNode>),
+    /// `IS NULL` expression
     SQLIsNull(Box<ASTNode>),
+    /// `IS NOT NULL` expression
     SQLIsNotNull(Box<ASTNode>),
+    /// Binary expression e.g. `1 + 1` or `foo > bar`
     SQLBinaryExpr {
         left: Box<ASTNode>,
         op: SQLOperator,
         right: Box<ASTNode>,
     },
+    /// CAST an expression to a different data type e.g. `CAST(foo AS VARCHAR(123))`
     SQLCast {
         expr: Box<ASTNode>,
         data_type: SQLType,
     },
+    /// Nested expression e.g. `(foo > bar)` or `(1)`
     SQLNested(Box<ASTNode>),
+    /// Unary expression
     SQLUnary {
         operator: SQLOperator,
         rex: Box<ASTNode>,
     },
+    /// Literal signed long
     SQLLiteralLong(i64),
+    /// Literal floating point value
     SQLLiteralDouble(f64),
+    /// Literal string
     SQLLiteralString(String),
-    SQLFunction {
-        id: String,
-        args: Vec<ASTNode>,
-    },
-    SQLOrderBy {
-        expr: Box<ASTNode>,
-        asc: bool,
-    },
+    /// Scalar function call e.g. `LEFT(foo, 5)`
+    SQLFunction { id: String, args: Vec<ASTNode> },
+    /// SELECT
     SQLSelect {
+        /// projection expressions
         projection: Vec<ASTNode>,
+        /// FROM
         relation: Option<Box<ASTNode>>,
+        /// WHERE
         selection: Option<Box<ASTNode>>,
-        order_by: Option<Vec<ASTNode>>,
+        /// ORDER BY
+        order_by: Option<Vec<SQLOrderByExpr>>,
+        /// GROUP BY
         group_by: Option<Vec<ASTNode>>,
+        /// HAVING
         having: Option<Box<ASTNode>>,
+        /// LIMIT
         limit: Option<Box<ASTNode>>,
     },
+    /// INSERT
+    SQLInsert {
+        /// TABLE
+        table_name: String,
+        /// COLUMNS
+        columns: Vec<String>,
+        /// VALUES (vector of rows to insert)
+        values: Vec<Vec<ASTNode>>,
+    },
+    /// UPDATE
+    SQLUpdate {
+        /// TABLE
+        table_name: String,
+        /// Column assignments
+        assignemnts: Vec<SQLAssigment>,
+        /// WHERE
+        selection: Option<Box<ASTNode>>,
+    },
+    /// DELETE
+    SQLDelete {
+        /// FROM
+        relation: Option<Box<ASTNode>>,
+        /// WHERE
+        selection: Option<Box<ASTNode>>,
+        /// ORDER BY
+        order_by: Option<Vec<SQLOrderByExpr>>,
+        limit: Option<Box<ASTNode>>,
+    },
+    /// CREATE TABLE
     SQLCreateTable {
         /// Table name
         name: String,
@@ -64,6 +109,26 @@ pub enum ASTNode {
     },
 }
 
+/// SQL assignment `foo = expr` as used in SQLUpdate
+#[derive(Debug, Clone, PartialEq)]
+pub struct SQLAssigment {
+    id: String,
+    value: Box<ASTNode>,
+}
+
+/// SQL ORDER BY expression
+#[derive(Debug, Clone, PartialEq)]
+pub struct SQLOrderByExpr {
+    pub expr: Box<ASTNode>,
+    pub asc: bool,
+}
+
+impl SQLOrderByExpr {
+    pub fn new(expr: Box<ASTNode>, asc: bool) -> Self {
+        SQLOrderByExpr { expr, asc }
+    }
+}
+
 /// SQL column definition
 #[derive(Debug, Clone, PartialEq)]
 pub struct SQLColumnDef {
