@@ -14,6 +14,7 @@
 
 //! SQL Parser
 
+use super::dialect::Dialect;
 use super::sqlast::*;
 use super::sqltokenizer::*;
 
@@ -51,8 +52,8 @@ impl Parser {
     }
 
     /// Parse a SQL statement and produce an Abstract Syntax Tree (AST)
-    pub fn parse_sql(sql: String) -> Result<ASTNode, ParserError> {
-        let mut tokenizer = Tokenizer::new(&sql);
+    pub fn parse_sql(dialect: &Dialect, sql: String) -> Result<ASTNode, ParserError> {
+        let mut tokenizer = Tokenizer::new(dialect, &sql);
         let tokens = tokenizer.tokenize()?;
         let mut parser = Parser::new(tokens);
         parser.parse()
@@ -87,6 +88,7 @@ impl Parser {
                     Token::Keyword(k) => match k.to_uppercase().as_ref() {
                         "SELECT" => Ok(self.parse_select()?),
                         "CREATE" => Ok(self.parse_create()?),
+                        "DELETE" => Ok(self.parse_delete()?),
                         _ => return parser_err!(format!("No prefix parser for keyword {}", k)),
                     },
                     Token::Mult => Ok(ASTNode::SQLWildcard),
@@ -440,6 +442,47 @@ impl Parser {
         }
     }
 
+    pub fn parse_delete(&mut self) -> Result<ASTNode, ParserError> {
+        let relation: Option<Box<ASTNode>> = if self.parse_keyword("FROM") {
+            Some(Box::new(self.parse_expr(0)?))
+        } else {
+            None
+        };
+
+        let order_by = if self.parse_keywords(vec!["ORDER", "BY"]) {
+            Some(self.parse_order_by_expr_list()?)
+        } else {
+            None
+        };
+
+        let limit = if self.parse_keyword("LIMIT") {
+            self.parse_limit()?
+        } else {
+            None
+        };
+
+        let selection = if self.parse_keyword("WHERE") {
+            Some(Box::new(self.parse_expr(0)?))
+        } else {
+            None
+        };
+
+        // parse next token
+        if let Some(next_token) = self.peek_token() {
+            parser_err!(format!(
+                "Unexpected token at end of DELETE: {:?}",
+                next_token
+            ))
+        } else {
+            Ok(ASTNode::SQLDelete {
+                relation,
+                selection,
+                order_by,
+                limit,
+            })
+        }
+    }
+
     /// Parse a SELECT statement
     pub fn parse_select(&mut self) -> Result<ASTNode, ParserError> {
         let projection = self.parse_expr_list()?;
@@ -519,8 +562,8 @@ impl Parser {
     }
 
     /// Parse a comma-delimited list of SQL ORDER BY expressions
-    pub fn parse_order_by_expr_list(&mut self) -> Result<Vec<ASTNode>, ParserError> {
-        let mut expr_list: Vec<ASTNode> = vec![];
+    pub fn parse_order_by_expr_list(&mut self) -> Result<Vec<SQLOrderByExpr>, ParserError> {
+        let mut expr_list: Vec<SQLOrderByExpr> = vec![];
         loop {
             let expr = self.parse_expr(0)?;
 
@@ -546,10 +589,7 @@ impl Parser {
                 None => true,
             };
 
-            expr_list.push(ASTNode::SQLOrderBy {
-                expr: Box::new(expr),
-                asc,
-            });
+            expr_list.push(SQLOrderByExpr::new(Box::new(expr), asc));
 
             if let Some(t) = self.peek_token() {
                 if t == Token::Comma {
@@ -580,6 +620,55 @@ impl Parser {
 mod tests {
 
     use super::*;
+    use super::super::dialect::GenericSqlDialect;
+
+    #[test]
+    fn parse_delete_statement() {
+        let sql: &str = "DELETE FROM 'table'";
+
+        match parse_sql(&sql) {
+            ASTNode::SQLDelete { relation, .. } => {
+                assert_eq!(
+                    Some(Box::new(ASTNode::SQLLiteralString("table".to_string()))),
+                    relation
+                );
+            }
+
+            _ => assert!(false),
+        }
+    }
+
+    #[test]
+    fn parse_where_delete_statement() {
+        let sql: &str = "DELETE FROM 'table' WHERE name = 5";
+
+        use self::ASTNode::*;
+        use self::SQLOperator::*;
+
+        match parse_sql(&sql) {
+            ASTNode::SQLDelete {
+                relation,
+                selection,
+                ..
+            } => {
+                assert_eq!(
+                    Some(Box::new(ASTNode::SQLLiteralString("table".to_string()))),
+                    relation
+                );
+
+                assert_eq!(
+                    SQLBinaryExpr {
+                        left: Box::new(SQLIdentifier("name".to_string())),
+                        op: Eq,
+                        right: Box::new(SQLLiteralLong(5)),
+                    },
+                    *selection.unwrap(),
+                );
+            }
+
+            _ => assert!(false),
+        }
+    }
 
     #[test]
     fn parse_simple_select() {
@@ -711,11 +800,11 @@ mod tests {
             ASTNode::SQLSelect { order_by, .. } => {
                 assert_eq!(
                     Some(vec![
-                        ASTNode::SQLOrderBy {
+                        SQLOrderByExpr {
                             expr: Box::new(ASTNode::SQLIdentifier("lname".to_string())),
                             asc: true,
                         },
-                        ASTNode::SQLOrderBy {
+                        SQLOrderByExpr {
                             expr: Box::new(ASTNode::SQLIdentifier("fname".to_string())),
                             asc: false,
                         },
@@ -841,14 +930,10 @@ mod tests {
         let sql = "SELECT 'one'";
         match parse_sql(&sql) {
             ASTNode::SQLSelect { ref projection, .. } => {
-                assert_eq!(
-                    projection[0],
-                    ASTNode::SQLLiteralString("one".to_string())
-                );
+                assert_eq!(projection[0], ASTNode::SQLLiteralString("one".to_string()));
             }
             _ => panic!(),
         }
-
     }
 
     #[test]
@@ -866,7 +951,8 @@ mod tests {
     }
 
     fn parse_sql(sql: &str) -> ASTNode {
-        let mut tokenizer = Tokenizer::new(&sql);
+        let dialect = GenericSqlDialect {};
+        let mut tokenizer = Tokenizer::new(&dialect,&sql, );
         let tokens = tokenizer.tokenize().unwrap();
         let mut parser = Parser::new(tokens);
         let ast = parser.parse().unwrap();
